/*
  povwrite.c

  write a povray colour map struct to
  a file or stream.

  J.J.Green 2005
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "pov.h"
#include "btrace.h"

extern int pov_write(const char* file,pov_t* pov)
{
  FILE *st;
  time_t t;
  int n,i;

  n = pov->n;

  if ( n<2 )
    {
      btrace("povray does not support %i stops",n);
      return 1;
    }

  if (file)
    {
      st = fopen(file,"w");
      if (!st) return 1;
    }
  else st = stdout;

  t = time(NULL);

  fprintf(st, "// autogenerated povray color map \"%s\"\n",
	  (pov->name ? pov->name : "[unnamed]"));
  fprintf(st, "// cptutils version %s, %s",VERSION,ctime(&t));
  
  if (pov->name)
    fprintf(st,"#declare %s =\n",pov->name);
  
  fprintf(st,"color_map {\n");

  for (i=0 ; i<n ; i++)
    {
      pov_stop_t stop = pov->stop[i];

      fprintf(st,"  [%7.5f color rgbf <%.4f,%.4f,%.4f,%.4f>]\n",
	      stop.z,
	      stop.rgbt[0],
	      stop.rgbt[1],
	      stop.rgbt[2],
	      stop.rgbt[3]);
    }

  fprintf(st,"}\n");

  fclose(st);

  return 0;
}

